using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Reflection;
using Microsoft.Win32;

using mshtml;

namespace Westwind.wwUtils
{
	/// <summary>
	/// Helper class for various Shell and String functions
	/// </summary>
	public class wwUtils
	{
//		public wwUtils()
//		{
//			//
//			// TODO: Add constructor logic here
//			//
//		}
			
		[DllImport("Shell32.dll")]
		private static extern int ShellExecute(int hwnd, string lpOperation, 
			string lpFile, string lpParameters, 
			string lpDirectory, int nShowCmd);

		/// <summary>
		/// Uses the Shell Extensions to launch a program based or URL moniker.
		/// </summary>
		/// <param name="lcUrl">Any URL Moniker that the Windows Shell understands (URL, Word Docs, PDF, Email links etc.)</param>
		/// <returns></returns>
		public static int GoUrl(string lcUrl)
		{
			string lcTPath = Environment.GetEnvironmentVariable("TEMP");
			int lnResult = ShellExecute(0,"open",lcUrl, "",lcTPath,1);
			return lnResult;
		}

		/// <summary>
		/// Returns the physical location of a given assembly. 
		/// </summary>
		/// <param name="loStartAssemblyObject">An object that is located in the startup assembly. If you leave this out the value is set to wwUtils (this class) which may or may not be located in the startup assembly.</param>
		/// <returns></returns>
		public static string GetAppStartPath(object loStartAssemblyObject) 
		{
			if (loStartAssemblyObject == null) 
				loStartAssemblyObject = new wwUtils();

			string lcStartupPath = loStartAssemblyObject.GetType().Assembly.Location;
			FileInfo loFile = new FileInfo(lcStartupPath);
			return loFile.DirectoryName;
		}

		/// <summary>
		/// Sets the Working Set (Memory Usage) of an application to the given size of bytes. 
		/// Will try to 'squeeze' process memory to the specified size by releasing unused code,
		/// specifically code for the JIT compiler. 
		/// 
		/// Recommended use is to call this method after all your code has been compiled to
		/// release the JIT compiler.
		/// </summary>
		/// <param name="lnMaxSize">Maximum memory used</param>
		/// <param name="lnMinSize">Minimum Memory used</param>
		public static void SetWorkingSet(int lnMaxSize,int lnMinSize) 
		{
			System.Diagnostics.Process loProcess = 	System.Diagnostics.Process.GetCurrentProcess();
			loProcess.MaxWorkingSet = (IntPtr) lnMaxSize;
			loProcess.MinWorkingSet = (IntPtr) lnMinSize;
			long lnValue = loProcess.WorkingSet;
		}

		#region Reflection Helper Code
		/// <summary>
		/// Binding Flags constant to be reused for all Reflection access methods.
		/// </summary>
		public const BindingFlags MemberAccess = 
			BindingFlags.Public | BindingFlags.NonPublic | 
			BindingFlags.Static | BindingFlags.Instance | BindingFlags.IgnoreCase ;

		/// <summary>
		/// Retrieve a dynamic 'non-typelib' property
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="lcProperty"></param>
		/// <returns></returns>
		public static object GetProperty(object loObject,string lcProperty)
		{
			return loObject.GetType().GetProperty(lcProperty,wwUtils.MemberAccess).GetValue(loObject,null);
		}

		/// <summary>
		/// Retrieve a dynamic 'non-typelib' field
		/// </summary>
		/// <param name="loObject"></param>
		/// <param name="lcProperty"></param>
		/// <returns></returns>
		public static object GetField(object loObject,string lcProperty)
		{
			return loObject.GetType().GetField(lcProperty,wwUtils.MemberAccess).GetValue(loObject);
		}

		/// <summary>
		/// Returns a property or field value using a base object and sub members including . syntax.
		/// For example, you can access: this.oCustomer.oData.Company with (this,"oCustomer.oData.Company")
		/// </summary>
		/// <param name="loParent">Parent object to 'start' parsing from.</param>
		/// <param name="lcProperty">The property to retrieve. Example: 'oBus.oData.Company'</param>
		/// <returns></returns>
		public static object GetPropertyEx(object loParent, string lcProperty) 
		{
			MemberInfo Member = null;

			Type loType = loParent.GetType();

			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0) 
			{
				if (lcProperty == "this" || lcProperty == "me")
					return loParent;

				// *** Get the member
				Member = loType.GetMember(lcProperty,wwUtils.MemberAccess)[0];
				if (Member.MemberType == MemberTypes.Property )
					return ((PropertyInfo) Member).GetValue(loParent,null);
				else
					return ((FieldInfo) Member).GetValue(loParent);
			}

			// *** Walk the . syntax - split into current object (lcMain) and further parsed objects (lcSubs)
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);

			// *** Retrieve the current property
			Member = loType.GetMember(lcMain,wwUtils.MemberAccess)[0];
			
			object loSub;
			if (Member.MemberType == MemberTypes.Property )
			{
				// *** Get its value
				loSub = ((PropertyInfo) Member).GetValue(loParent,null);

			}
			else
			{
				loSub = ( (FieldInfo) Member).GetValue(loParent);

			}

			// *** Recurse further into the sub-properties (lcSubs)
			return wwUtils.GetPropertyEx(loSub,lcSubs);
		}

		public static void SetProperty(object loObject,string lcProperty,object loValue)
		{
			loObject.GetType().GetProperty(lcProperty,wwUtils.MemberAccess).SetValue(loObject,loValue,null);
		}

		public static void SetField(object loObject,string lcProperty,object loValue)
		{
			loObject.GetType().GetField(lcProperty,wwUtils.MemberAccess).SetValue(loObject,loValue);
		}

		/// <summary>
		/// Sets the value of a field or property via Reflection. This method allows 
		/// for using '.' syntax to specify objects multiple levels down.
		/// 
		/// Example:
		/// <<pre>>wwUtils.SetPropertyEx(this,"Invoice.LineItemsCount",10)<</pre>>
		/// 
		/// which would be equivalent of:
		/// 
		/// <<pre>>this.Invoice.LineItemsCount = 10;<</pre>>
		/// <seealso>Class wwUtils</seealso>
		/// </summary>
		/// <param name="Object loParent">
		/// Object to set the property on.
		/// </param>
		/// <param name="String lcProperty">
		/// Property to set. Can be an object hierarchy with . syntax.
		/// </param>
		/// <param name="Object loValue">
		/// Value to set the property to
		/// </param>
		public static object SetPropertyEx(object loParent, string lcProperty,object loValue) 
		{
			Type loType = loParent.GetType();
			MemberInfo Member = null;

			// *** no more .s - we got our final object
			int lnAt = lcProperty.IndexOf(".");
			if ( lnAt < 0) 
			{
				Member = loType.GetMember(lcProperty,wwUtils.MemberAccess)[0];
				if ( Member.MemberType == MemberTypes.Property ) 
				{

					((PropertyInfo) Member).SetValue(loParent,loValue,null);
					return null;
				}
				else 
				{
					((FieldInfo) Member).SetValue(loParent,loValue);
					return null;				   
				}
			}	
				
			// *** Walk the . syntax
			string lcMain = lcProperty.Substring(0,lnAt);
			string lcSubs = lcProperty.Substring(lnAt+1);
			Member = loType.GetMember(lcMain,wwUtils.MemberAccess)[0];

			object loSub;
			if (Member.MemberType == MemberTypes.Property)
				loSub = ((PropertyInfo) Member).GetValue(loParent,null);
			else
				loSub = ((FieldInfo) Member).GetValue(loParent);

			// *** Recurse until we get the lowest ref
			SetPropertyEx(loSub,lcSubs,loValue);
			return null;
		}

		/// <summary>
		/// Wrapper method to call a 'dynamic' (non-typelib) method
		/// on a COM object
		/// </summary>
		/// <param name="loParams"></param>
		/// 1st - Method name, 2nd - 1st parameter, 3rd - 2nd parm etc.
		/// <returns></returns>
		public static object CallMethod(object loObject,string lcMethod, params object[] loParams)
		{
			return loObject.GetType().GetMethod(lcMethod,wwUtils.MemberAccess).Invoke(loObject,loParams);
		}

		#endregion

		public static void OpenHelpBuilder(ref object loHelp,string lcHelpId) 
		{
			if (loHelp == null) 
			{
				Type loT = Type.GetTypeFromProgID("wwHelp.wwHelp");
				loHelp =  Activator.CreateInstance(loT);
			}

			if (lcHelpId != null && lcHelpId.Length > 0) 
			{
				int lnIndex = lcHelpId.LastIndexOf(".");
				if (lnIndex > 0)
					lcHelpId = lcHelpId.Substring(0,lcHelpId.LastIndexOf("."));

				wwUtils.CallMethod(loHelp,"OpenForm",lcHelpId,0);
			}
		}

		public static string FileToString(string lcFilename) 
		{
			string lcResult;

			try 
			{
				StreamReader loStream = new StreamReader(lcFilename);
				lcResult = loStream.ReadToEnd();
				loStream.Close();
			}
			catch(Exception) 
			{
				return "";
			}

			return lcResult;
		}		

		public static bool StringToFile(string lcString, string lcFilename) 
		{
			try 
			{
				StreamWriter loStream = new StreamWriter(lcFilename,false);
				loStream.Write(lcString);
				loStream.Close();
			}
			catch(Exception) 
			{
				return false;
			}

			return true;
		}	
	
		/// <summary>
		/// Extracts a string from between a pair of delimiters. Only the first 
		/// instance is found.
		/// <seealso>Class wwUtils</seealso>
		/// </summary>
		/// <param name="Source">
		/// Input String to work on
		/// </param>
		/// <param name="StartDelim">
		/// Beginning delimiter
		/// </param>
		/// <param name="EndDelim">
		/// ending delimiter
		/// </param>
		/// <param name="CaseInsensitive">
		/// Determines whether the search for delimiters is case sensitive
		/// </param>
		/// <returns>Extracted string or ""</returns>
		public static string ExtractString(string Source, string BeginDelim, string EndDelim, bool CaseInSensitive) 
		{
			int At1, At2;

			
			if (CaseInSensitive) 
			{
				At1 = Source.IndexOf(BeginDelim);
				At2 = Source.IndexOf(EndDelim,At1+ BeginDelim.Length + 1);
			}
			else 
			{
				string Lower = Source.ToLower();
				At1 =Lower.IndexOf( BeginDelim.ToLower() );
				At2 = Lower.IndexOf( EndDelim.ToLower(),At1 + BeginDelim.Length + 1 );
			}
			  
			if (At1 > -1 && At2 > 1) 
			{
				return Source.Substring(At1 + BeginDelim.Length,At2-At1 - BeginDelim.Length);
			}

			return "";
		}

		/// <summary>
		/// Extracts a string from between a pair of delimiters. Only the first 
		/// instance is found. Search is case insensitive.
		/// <seealso>Class wwUtils</seealso>
		/// </summary>
		/// <param name="Source">
		/// Input String to work on
		/// </param>
		/// <param name="StartDelim">
		/// Beginning delimiter
		/// </param>
		/// <param name="EndDelim">
		/// ending delimiter
		/// </param>
		/// <returns>Extracted string or ""</returns>
		public static string ExtractString(string Source, string BeginDelim, string EndDelim) 
		{
			return wwUtils.ExtractString(Source,BeginDelim,EndDelim,false);
		}

		/// <summary>
		/// Adds a specific extension to a filename
		/// </summary>
		/// <param name="lcFilename">Filename to add extension to</param>
		/// <param name="lcExtension">Extension (without the .) to add</param>
		/// <returns>filename with the specified extension</returns>
		public static string ForceExtension(string lcFilename, string lcExtension) 
		{
			int lnIndex = lcFilename.LastIndexOf(".");
			if (lnIndex == -1) 
			{
				return lcFilename + "." + lcExtension;
			}
			return lcFilename.Substring(0,lnIndex + 1) + lcExtension;
		}
	}

	
}